unit MDB_Defines;

(******************************************************************************
 *                                                                            *
 *   wh Mnzprfer Dietmar Trenner GmbH                                       *
 *                                                                            *
 *   Teltower Damm 276                                                        *
 *   D-14167 Berlin                                                           *
 *                                                                            *
 *   Accessing MDB devices under Windows and Linux                            *
 *                                                                            *
 *   Definitions of constants and structures                                  *
 *                                                                            *
 *   Version 0.10, 06.03.2003                                                 *
 *                                                                            *
 ******************************************************************************)

interface

(**************************************************************** definitions *)

const	BUFFSIZE		=  128;			// size of in-/outputbuffer
		MAXBLKLEN		=   40;			// max. block length

(*************************************************** error codes MDB handling *)

const	MDB_OK			=  0;			// max. block length
		MDB_BADCOMNO	= -1;			// invalid COM port
		MDB_NOINIT		= -2;			// not initialised
		MDB_OPENERROR   = -3;			// can't open COM
		MDB_INITERROR   = -4;			// can't initialise COM
		MDB_BADCSUM		= -5;			// checksum error
		MDB_ANSTO		= -6;			// response timeout
		MDB_CHARTO		= -7;			// character timeout
		MDB_NOACK		= -8;			// no Acknowledge
		MDB_NAKRCVD		= -9;			// NAK received
		MDB_NOTRDY	    =-10;			// Controller not ready
		MDB_TOOLONG		=-11;			// block too long
		MDB_SNDERROR	=-12;			// COM send error
		MDB_RCVERROR	=-13;			// COM receive error
		MDB_CTRLTOT		=-14;           // Controller timeout
		MDB_WRONGADR	=-15;			// invalid MDB address
		MDB_WRONGMC		=-16;			// wrong Controller
		MDB_BADID		=-20;			// can't identify device
		MDB_DLLERR		=-21;			// error loading DLL
        MDB_PROCERR		=-22;			// error getting procedure address


(************************************************ error codes coin validator *)

const	EMP_OK			=   0;
 		EMP_NOINIT	  	=-101;			// coin validator not open
		EMP_OPENED		=-102;			// coin validator already opened
 		EMP_ANSERROR 	=-103;			// coin validator didn't respond
 		EMP_NULLPTR   	=-104;			// call with invalid pointer
		EMP_BADLEN		=-105;			// coin validator wrong block length in response

// Pollstatus
const 	EMPP_COIN		= $00000001;	// coin data transmitted
 		EMPP_RESET		= $00000002;	// was reset
 		EMPP_RETURN		= $00000004;	// return lever pushed
		EMPP_REJECT		= $00000008;	// coin rejected
		EMPP_NOINIT		= $00000010;	// not initialised
		EMPP_NOANSWER	= $00000020;	// no response
		EMPP_UNKNOWN	= $00000040;	// unknown status
        EMPP_TIMEOUT	= $00000080;	// No-Response time exceeded

		EMPP_MDBERR		= $10000000;	// MDB error

(******************************************************** error codes changer *)

const	CHG_OK			=   0;
 		CHG_NOINIT	  	=-201;			// changer not open
		CHG_OPENED		=-202;			// changer already opened
 		CHG_ANSERROR 	=-203;			// changer didn't respond
 		CHG_NULLPTR   	=-204;			// call with invalid pointer
		CHG_BADLEN		=-205;			// wrong block length in response
		CHG_BADLEVEL	=-206;			// insufficient featurer level
		CHG_DISPERR		=-207;			// payout error
        CHG_INTERNAL    =-208;          // internal error 

// bit masks for polling results
const 	CHGP_DISPENSE	= $00000001;	// coin dispensed
		CHGP_DEPOSIT	= $00000002;	// coin accepted
		CHGP_RETURN		= $00000004;    // return lever pushed
		CHGP_POTBUSY	= $00000008;	// payout busy
		CHGP_NOCREDIT	= $00000010;	// error accepting coin
		CHGP_SENSFAIL	= $00000020;	// defective tube sensor
		CHGP_DBLARRVL	= $00000040;	// followup coin
		CHGP_UNPLGGD	= $00000080;	// coin validator removed
		CHGP_TUBEJAM	= $00000100;	// tube jam
		CHGP_ROMCSUM	= $00000200;	// internal checksum error
		CHGP_ROUTERR	= $00000400;	// internal routing error
		CHGP_BUSY		= $00000800;	// busy
		CHGP_RESET		= $00001000;	// was reset
		CHGP_COINJAM	= $00002000;	// coin jam
		CHGP_SLUG		= $00004000;	// slug
		CHGP_REJECT		= $00008000;	// rejected
        CHGP_TIMEOUT	= $00010000;	// No-Response time exceeded
		CHGP_UNKNOWN	= $00080000;	// unknown status

		CHGP_MDBERR		= $10000000;	// MDB error

// not fully tested: attributes of the changer's display
const	CHGD_CLEAR		= $00000080;	// vor Ausgabe lschen
		CHGD_SCROLL		= $00000001;	// Scrollen
		CHGD_FLASH		= $00000002;	// Blinken

(************************************************* error codes bill validator *)

const	BLV_OK			=   0;
		BLV_NOINIT	  	=-301;			// bill validator not open
		BLV_OPENED		=-302;			// bill validator already opened
		BLV_ANSERROR 	=-303;			// bill validator didn't respond
		BLV_NULLPTR   	=-304;			// call with invalid pointer
		BLV_BADLEN		=-305;			// wrong block length in response
		BLV_BADLEVEL	=-306;			// insufficient feature level
		BLV_ESCRERR		=-307;			// escrow error
		BLV_STACKERR	=-308;			// stack error

// bill position
const	NO_BILL			= 100;			// no bill
		BILL_STACKED	=   0;			// bill stacked
		ESCROW_POSITION =   1;			// bill in escrow position
		BILL_RETURNED	=   2;			// bill retrurned
		NOT_USED		=   3;			// not used
		BILL_REJECTED	=	4;			// disabled bill rejected

// poll status
const   BLVP_BILL		= $00000001;	// bill accepted
		BLVP_MOTFAIL	= $00000002;	// motor error
		BLVP_SENSFAIL	= $00000004;	// sensor error
		BLVP_BUSY		= $00000008;	// busy
		BLVP_ROMCSUM	= $00000010;	// ROM checksum error
		BLVP_JAMMED		= $00000020;	// bill jam
		BLVP_RESET		= $00000040;	// was reset
		BLVP_REMOVED	= $00000080;	// bill manually removed
		BLVP_NOCASHBOX	= $00000100;	// cash box removed
		BLVP_DISABLED	= $00000200;	// disabled
		BLVP_BADESCROW	= $00000400;	// invalid escrow request
		BLVP_REJECTED	= $00000800;	// bill returned
		BLVP_TIMEOUT	= $00001000;	// No-Response time exceeded
		BLVP_UNKNOWN	= $00080000;	// Who knows?

		BLVP_MDBERR		= $10000000;	// MDB error

(**************************************************** error codes card reader *)

const	CRD_OK			=   0;
		CRD_NOINIT	  	=-401;			// card reader not open
		CRD_OPENED		=-402;			// card reader already opened
		CRD_ANSERROR 	=-403;			// card reader didn't respond
		CRD_NULLPTR   	=-404;			// call with invalid pointer
		CRD_BADLEN		=-405;			// wrong block length in response
		CRD_BADLEVEL	=-406;			// wrong feature level
		CRD_RANGE		=-407;			// amount out of range
		CRD_SEQUERROR	=-408;			// command out of sequence

		CRD_CARDERROR	=-420;			// card error
		CRD_INVALID		=-421;			// invalid card
		CRD_TAMPER		=-422;			// tamper error
		CRD_MANDEF1		=-423;			// manufacturer defined error
		CRD_COMMERR1	=-424;			// communications error 1
		CRD_SERVICE		=-425;			// reader requires service
		CRD_UNASSGND	=-426;			// unassigned error
		CRD_MANDEF2		=-427;			// manufacturer defined error
		CRD_READER		=-428;          // reader failure
		CRD_COMMERR2	=-429;			// communications error 2
		CRD_CARDJAM		=-430;			// card jammed
		CRD_MANDEF3		=-431;			// manufacturer defined error
		CRD_REFUND		=-432;			// refund error

// card reader states
		CRDS_CLOSED			= -1;		// not yet opened
		CRDS_INACTIVE		=  0;
		CRDS_DISABLED		=  1;
		CRDS_ENABLED		=  2;
		CRDS_SESSION		=  3;
		CRDS_VEND			=  4;

//card reader pollresults
		CRDP_RESET		= $00000001;	// just reset
		CRDP_CONFIG		= $00000002;	// responding to CONFIGURATION
		CRDP_DISPLAY	= $00000004;	// display request
		CRDP_BEGIN      = $00000008;	// begin session
		CRDP_CANCELREQ	= $00000010;	// session cancel request
		CRDP_VENDAPPRVD	= $00000020;	// vend approved
		CRDP_VENDDENIED	= $00000040;	// vend denied
		CRDP_END		= $00000080;	// end session
		CRDP_CANCELED	= $00000100;	// cancel session
		CRDP_IDENTIFY	= $00000200;	// identify
		CRDP_ERROR		= $00000400;	// error report
		CRDP_SEQUENCE	= $00000800;	// command out of sequence
		CRDP_BUSY		= $00001000;	// reader busy
		CRDP_REVAPPRVD	= $00002000;	// revalue approved
		CRDP_REVDENIED	= $00004000;	// revalue denied
		CRDP_REVLIMIT	= $00008000;	// revalue limit
		CRDP_TIMEOUT	= $01000000;	// No-Response time exceeded
		CRDP_UNKNOWN	= $08000000;	// Who knows?
		CRDP_MDBERR		= $10000000;	// mdb error (don't care)


(************************************************************ type definitons *)

(***************************************************************** MDB common *)

// devices at the bus
type MDBEntry = record
		exist:			Integer;					// device exists
		supported:		Integer;					// supported by DLL
		manufacturer:	Array [0.. 3] of Char;		// manufacturer
		model:			Array [0..12] of Char;		// model
	end;
	lpMDBEntry	= ^MDBEntry;

// pointers
type	DoublePtr	= ^Double;
		IntegerPtr	= ^Integer;

(************************************************************* coin validator *)

// about the coin validator
type EMPInfo = record
{$IFDEF LINUX}
		manufacturer:   Array [0.. 3] of Char;      // manufacturer
{$ENDIF}
		device,										// device name
		model:			Array [0..12] of Char;  	// model
		version:		Array [0.. 1] of Byte;		// software version
		features:		Array [0.. 3] of Byte;		// feature flags
		country:		Integer;					// country code
		decimals:		Integer;					// decimals
		coin_value:		Array [0..15] of Single;	// coin values
		tubes,										// number of slots
		cashbox:		Integer;					// number of the cashbox
		tube:   		Array [0..15] of Integer;	// coin routing
end;
	 EMPInfoPtr	= ^EMPInfo;

// coin routing and locking
type EMPCurrent = record
		cashbox:		Integer;					// cashbox
		unlock,										// unlock coin
		incbox,										// route coin to cashbox
		tube:			Array [0..15] of Integer;	// coin slot
end;
	 EMPCurrentPtr	= ^EMPCurrent;

// last poll status
type EMPPollStatus = record
	status, coin, slot, reject, extended, mdberr:	Integer;
end;
	EMPPollStatusPtr	= ^EMPPollStatus;

(*************************************************************** change giver *)

// about the change giver
type CHGInfo = record
{$IFDEF LINUX}
		manufacturer:   Array [0.. 3] of Char;      // manufacturer
{$ENDIF}
		device,										// device name
		model:			Array [0..12] of Char;  	// model
		version:		Array [0.. 1] of Byte;		// software version
		features:		Array [0.. 3] of Byte;		// feature flags
		country:		Integer;					// country code
		decimals:		Integer;					// decimals
		least_coin:		Single;						// least dispensable coin
		coin_value:		Array [0..15] of Single;	// coin values
		tube_coin:		Array [0..15] of Integer;	// flags for tube coins
end;
     CHGInfoPtr	= ^CHGInfo;

// coin locking
type CHGCurrent  	= Array [0..15] of Integer;		// unlock
	 CHGCurrentPtr	= ^CHGCurrent;

// tube status
type CHGTubeStat = record
		coinctr,									// number of coins
		tube_full,									// tube full
		tube_err:		Array [0..15] of Integer;	// defective tube
end;
	CHGTubeStatPtr	= ^CHGTubeStat;

// dispensed coins
type CHGDispense  	= Array [0..15] of Integer;		// number of coins
	 CHGDispensePtr	= ^CHGDispense;

// last poll status
type CHGPollStatus = record
	status, coin, coinctr, mdberr:	Integer;
end;
	CHGPollStatusPtr	= ^CHGPollStatus;

(************************************************************* bill validator *)

// about the bill validator
type BLVInfo = record
{$IFDEF LINUX}
		manufacturer:   Array [0.. 3] of Char;      // manufacturer
{$ENDIF}
		device,										// device name
		model:			Array [0..12] of Char;  	// model
		version:		Array [0.. 1] of Byte;		// software version
		features:		Array [0.. 3] of Byte;		// feature flags
		country:		Integer;					// country code
		decimals:		Integer;					// decimals
		escrowcap:		Integer;					// escrow capability
		stackercap:		Integer;					// stacke capacity
		bill_value:		Array [0..15] of Single;	// bill values
end;
	 BLVInfoPtr	= ^BLVInfo;

// bill locking
type BLVCurrent  = record
		unlock,										// unlock
		escrenab:		Array [0..15] of Integer;	// escrow enabled
end;
	 BLVCurrentPtr	= ^BLVCurrent;

// high bill security
type BLVSecurity = Array [0..15] of Integer;

	 BLVSecurityPtr = ^BLVSecurity;

// stacker status
type BLVStacker = record
	bills, full:		Integer;					// number of bills, full status
end;
	BLVStackerPtr = ^BLVStacker;

// last poll status
type BLVPollStatus = record
	status, bill, billpos, mdberr:	Integer;
end;
	BLVPollStatusPtr	= ^BLVPollStatus;


(**************************************************************** card reader *)

// about the card reader
type CRDInfo = record
{$IFDEF LINUX}
		manufacturer:   Array [0.. 3] of Char;      // manufacturer
{$ENDIF}
		country:		Integer;					// country code
		decimals:		Integer;					// decimals
		device,										// device name
		model:			Array [0..12] of Char;  	// model
		version:		Array [0.. 1] of Byte;		// software version
		canreval,									// can revalue
		canmulti,									// multivend capable
		havedisp:		Integer;					// have display
		revallim:		Single;						// revalue limit
end;
	 CRDInfoPtr	= ^CRDInfo;

// card readersetup data
type CRDSetup = record
		maxprice,									// maximum price
		minprice:		Single;						// minimum price
		enabled:		Integer                     // state
end;
	CRDSetupPtr		= ^CRDSetup;

// card reader display data
type CRDDisplay = Record
		lines:		Array [0..1, 0..15] of Char;    // display data
		expire:     Integer;                        // expire time in milliseconds
end;
	CRDDisplayPtr	= ^CRDDisplay;

// payment media data
type CRDMediaInfo = Record
		mid:			Cardinal;					// media id
		mtype,										// media type: 0 - normal, 1 - test
		mfree:			Integer;					// 1 - free vend card
		vtype,										// vend type
		allocation,                                 // cost allocation code
		index:			Integer;                    // number/index
		surdis:			Double;						// surcharge/discount
end;

	CRDMediaInfoPtr	= ^CRDMediaInfo;

// last poll status
type CRDPollStatus = record
		status,
		crderror,             						// card reader error status
		mdberror:		Integer;                    // interface error status
end;
	CRDPollStatusPtr	= ^CRDPollStatus;

(************************************************************** miscellaneous *)

type
	InOutBuff		= Array [0..BUFFSIZE] of Byte;	// in-/ouput buffer...
	InOutBuffPtr	= ^InOutBuff;					// ... and pointer

implementation

end.
