/******************************************************************************
 *                                                                            *
 *   wh Mnzprfer Berlin GmbH                                                *
 *                                                                            *
 *   Teltower Damm 276                                                        *
 *   D-14167 Berlin                                                           *
 *                                                                            *
 *   Accessing MDB and ccTalk devices under Windows 98/ME/2000/XP             *
 *                                                                            *
 *   Definitions                                                              *
 *                                                                            *
 *   Version 3.00, 17.11.2005                                                 *
 *                                                                            *
 ******************************************************************************/

/**************************************************************** definitions */

#ifndef _WINMDB32_H
#define _WINMDB32_H

#define	BUFFSIZE		 128			// size of in-/outputbuffer
#define	MAXBLKLEN		 40				// max. block length

/***************************************************************** open modes */

#define	OM_NOCOMIF		0x00010000;		// don't check for COM interface
#define	OM_NOCCTIF		0x00020000;		// don't check for CCT interface

/*************************************************** error codes MDB handling */

#define	MDB_OK			 0				// O.K.
#define MDB_BADCOMNO	-1				// invalid COM port
#define MDB_NOINIT		-2				// not initialised
#define MDB_OPENERROR   -3				// can't open COM
#define MDB_INITERROR   -4				// can't initialise COM
#define MDB_BADCSUM		-5				// checksum error
#define MDB_ANSTO		-6				// response timeout
#define MDB_CHARTO      -7				// character timeout
#define MDB_NOACK		-8				// no Acknowledge
#define MDB_NAKRCVD		-9				// NAK received
#define MDB_NOTRDY	    -10				// Controller not ready
#define MDB_TOOLONG		-11				// block too long
#define MDB_SNDERROR	-12				// COM send error
#define MDB_RCVERROR	-13				// COM receive error
#define MDB_CTRLTOT		-14    			// Controller timeout
#define MDB_WRONGADR	-15				// invalid MDB address
#define MDB_WRONGMC		-16				// wrong Controller
#define MDB_BADID		-20				// can't identify device
#define MDB_DLLERR		-21;			// error loading DLL
#define MDB_PROCERR		-22;			// error getting procedure address


/************************************************ error codes coin validator */

#define EMP_OK			   0
#define EMP_NOINIT	  	-101			// coin validator not open
#define EMP_OPENED		-102			// coin validator already opened
#define EMP_ANSERROR 	-103			// coin validator didn't respond
#define EMP_NULLPTR   	-104			// call with invalid pointer
#define EMP_BADLEN		-105			// coin validator wrong block length in response

// poll status
#define	EMPP_COIN		0x00000001		// coin data transmitted
#define EMPP_RESET		0x00000002		// was reset
#define EMPP_RETURN		0x00000004		// return lever pushed
#define EMPP_REJECT		0x00000008		// coin rejected
#define EMPP_NOINIT		0x00000010		// not initialised
#define EMPP_NOANSWER	0x00000020		// no response
#define EMPP_UNKNOWN	0x00000040		// unknown status
#define EMPP_TIMEOUT	0x00000080		// No-Response time exceeded

#define EMPP_MDBERR		0x10000000		// MDB error

/******************************************************** error codes changer */

#define	CHG_OK			   0
#define CHG_NOINIT	  	-201			// changer not open
#define CHG_OPENED		-202			// changer already opened
#define CHG_ANSERROR 	-203			// changer didn't respond
#define CHG_NULLPTR   	-204			// call with invalid pointer
#define CHG_BADLEN		-205			// wrong block length in response
#define CHG_BADLEVEL	-206			// insufficient featurer level
#define CHG_DISPERR		-207			// payout error
#define CHG_INTERNAL    -208;          // internal error

// bit masks for poll results
#define CHGP_DISPENSE	0x00000001		// coin dispensed
#define CHGP_DEPOSIT	0x00000002		// coin accepted
#define CHGP_RETURN		0x00000004    	// return lever pushed
#define CHGP_POTBUSY	0x00000008		// payout busy
#define CHGP_NOCREDIT	0x00000010		// error accepting coin
#define CHGP_SENSFAIL	0x00000020		// defective tube sensor
#define CHGP_DBLARRVL	0x00000040		// followup coin
#define CHGP_UNPLGGD	0x00000080		// coin validator removed
#define CHGP_TUBEJAM	0x00000100		// tube jam
#define CHGP_ROMCSUM	0x00000200		// internal checksum error
#define CHGP_ROUTERR	0x00000400		// internal routing error
#define CHGP_BUSY		0x00000800		// busy
#define CHGP_RESET		0x00001000		// was reset
#define CHGP_COINJAM	0x00002000		// coin jam
#define CHGP_SLUG		0x00004000		// slug
#define CHGP_REJECT		0x00008000		// rejected
#define CHGP_TIMEOUT	0x00010000		// No-Response time exceeded
#define CHGP_UNKNOWN	0x00080000		// unknown status

#define CHGP_MDBERR		0x10000000		// MDB error

/************************************************* error codes bill validator */

#define	BLV_OK			   0
#define BLV_NOINIT	  	-301			// bill validator not open
#define BLV_OPENED		-302			// bill validator already opened
#define BLV_ANSERROR 	-303			// bill validator didn't respond
#define BLV_NULLPTR   	-304			// call with invalid pointer
#define BLV_BADLEN		-305			// wrong block length in response
#define BLV_BADLEVEL	-306			// insufficient feature level
#define BLV_ESCRERR		-307			// escrow error
#define BLV_STACKERR	-308			// stack error

// bill position
#define	NO_BILL			  100			// no bill
#define	BILL_STACKED	   	0			// bill stacked
#define	ESCROW_POSITION  	1			// bill in escrow position
#define	BILL_RETURNED	   	2			// bill retrurned
#define	NOT_USED		    3			// not used
#define	BILL_REJECTED	   	4			// disabled bill rejected

// poll status
#define BLVP_BILL	    0x00000001		// bill accepted
#define BLVP_MOTFAIL	0x00000002		// motor error
#define BLVP_SENSFAIL	0x00000004		// sensor error
#define BLVP_BUSY		0x00000008		// busy
#define BLVP_ROMCSUM	0x00000010		// ROM checksum error
#define BLVP_JAMMED		0x00000020		// bill jam
#define BLVP_RESET		0x00000040		// was reset
#define BLVP_REMOVED	0x00000080		// bill manually removed
#define BLVP_NOCASHBOX	0x00000100		// cash box removed
#define BLVP_DISABLED	0x00000200		// disabled
#define BLVP_BADESCROW	0x00000400		// invalid escrow request
#define BLVP_REJECTED	0x00000800		// bill returned
#define BLVP_TIMEOUT	0x00001000		// No-Response Zeit exceeded
#define BLVP_UNKNOWN	0x00080000		// Who knows?

#define BLVP_MDBERR		0x10000000		// MDB error

/**************************************************** error codes card reader *)

#define CRD_OK				   0;
#define CRD_NOINIT	  		-401;			// card reader not open
#define CRD_OPENED			-402;			// card reader already opened
#define CRD_ANSERROR 		-403;			// card reader didn't respond
#define CRD_NULLPTR   		-404;			// call with invalid pointer
#define CRD_BADLEN			-405;			// wrong block length in response
#define CRD_BADLEVEL		-406;			// wrong feature level
#define CRD_RANGE			-407;			// amount out of range
#define CRD_SEQUERROR		-408;			// command out of sequence

#define CRD_CARDERROR		-420;			// card error
#define CRD_INVALID			-421;			// invalid card
#define CRD_TAMPER			-422;			// tamper error
#define CRD_MANDEF1			-423;			// manufacturer defined error
#define CRD_COMMERR1		-424;			// communications error 1
#define CRD_SERVICE			-425;			// reader requires service
#define CRD_UNASSGND		-426;			// unassigned error
#define CRD_MANDEF2			-427;			// manufacturer defined error
#define CRD_READER			-428;          	// reader failure
#define CRD_COMMERR2		-429;			// communications error 2
#define CRD_CARDJAM			-430;			// card jammed
#define CRD_MANDEF3			-431;			// manufacturer defined error
#define CRD_REFUND			-432;			// refund error

// card reader states
#define CRDS_CLOSED				 -1;		// not yet opened
#define CRDS_INACTIVE			  0;
#define CRDS_DISABLED			  1;
#define CRDS_ENABLED			  2;
#define CRDS_SESSION			  3;
#define CRDS_VEND				  4;

//card reader pollresults
#define CRDP_RESET			 $00000001;		// just reset
#define CRDP_CONFIG			 $00000002;		// responding to CONFIGURATION
#define CRDP_DISPLAY		 $00000004;		// display request
#define CRDP_BEGIN      	 $00000008;		// begin session
#define CRDP_CANCELREQ		 $00000010;		// session cancel request
#define CRDP_VENDAPPRVD		 $00000020;		// vend approved
#define CRDP_VENDDENIED		 $00000040;		// vend denied
#define CRDP_END			 $00000080;		// end session
#define CRDP_CANCELED		 $00000100;		// cancel session
#define CRDP_IDENTIFY		 $00000200;		// identify
#define CRDP_ERROR			 $00000400;		// error report
#define CRDP_SEQUENCE		 $00000800;		// command out of sequence
#define CRDP_BUSY			 $00001000;		// reader busy
#define CRDP_REVAPPRVD		 $00002000;		// revalue approved
#define CRDP_REVDENIED		 $00004000;		// revalue denied
#define CRDP_REVLIMIT		 $00008000;		// revalue limit
#define CRDP_TIMEOUT		 $01000000;		// No-Response time exceeded
#define CRDP_UNKNOWN		 $08000000;		// Who knows?
#define CRDP_MDBERR			 $10000000;		// mdb error (don't care)


/************************************************************ type definitons */

/************************************************************* coin validator */

// about the coin validator
typedef struct {
		char			device[13],					// device name
						model[13];				  	// model
		unsigned char	version[2],					// software version
						features[4];				// feature flags
		int				country,					// country code
						decimals;					// decimals
		float			coin_value[16];				// coin values
		int				tubes,						// number of slots
						cashbox,					// number of the cashbox
						tube[16];					// coin routing
} EMPInfo;

// more about the coin validator
typedef struct {
		char            manufacturer[33],			// manufacturer
        				device[13],					// device name
						model[13];				  	// model
		unsigned char	version[2],					// software version
						features[4];				// feature flags
		int				country,					// country code
						decimals;					// decimals
		float			coin_value[16];				// coin values
        char			coin_currency[16][3];		// currency ID
		int				tubes,						// number of slots
						cashbox,					// number of the cashbox
						tube[16];					// coin routing
} EMPInfoEx;

// coin routing and locking
typedef struct {
		int				cashbox,					// cashbox
						unlock[16],					// unlock coin
						incbox[16],					// route coin to cashbox
						tube[16];					// coin slot
} EMPCurrent;

// last poll status
typedef struct {
	int					status,
						coin, slot, reject,
						extended, mdberr;
} EMPPollStatus;

/*************************************************************** change giver */

// about the change giver
typedef struct {
		char			device[13],					// device name
						model[13];					// model
		unsigned char	version[2],					// software version
						features[4];				// feature flags
		int				country,					// country code
						decimals;					// decimals
		float			least_coin,					// least dispensable coin
						coin_value[16];				// coin values
		int				tube_coin[16];				// flags for tube coins
} CHGInfo;

// coin locking
typedef int				CHGCurrent[16];				// unlock

// tube status
typedef struct {
		int				coinctr[16],				// number of coins
						tube_full[16],				// tube full
						tube_err[16];				// defective tube
} CHGTubeStat;

// dispensed coins
typedef int 			CHGDispense[16];			// number of coins

// last poll status
typedef struct {
		int				status,
						coin, coinctr,
						mdberr;
} CHGPollStatus;

/************************************************************* bill validator */

// about the bill validator
typedef struct {
		char			device[13],					// device name
						model[13];					// model
		unsigned char	version[2],					// software version
						features[4];				// feature flags
		int				country,					// country code
						decimals,					// decimals
						escrowcap,					// escrow capability
						stackercap;					// stacke capacity
		float			bill_value[16];					// bill values
} BLVInfo;

// bill locking
typedef struct {
		int				unlock[16],					// unlock
						escrenab[16];				// escrow enabled
} BLVCurrent;

// stacker status
typedef struct {
		int				bills, full;				// number of bills, full status
} BLVStacker;

// last poll status
typedef struct {
		int				status,
						bill, billpos,
						mdberr;
} BLVPollStatus;

/**************************************************************** card reader */

// about the card reader
typedef struct CRDInfo {
		int				country,					// country code
						decimals;					// decimals
		char			device[13],					// device name
						model[13];			  		// model
		unsigned char	version[2];					// software version
		int				canreval,					// can revalue
						canmulti,					// multivend capable
						havedisp;					// have display
		float			revallim;					// revalue limit
} CRDInfo;

// card readersetup data
typedef struct {
		float			maxprice,					// maximum price
						minprice;					// minimum price
		int				enabled;                    // state
} CRDSetup;

// card reader display data
typedef struct {
		char			lines[2][16];    			// display data
		int				expire;                     // expire time in milliseconds
} CRDDisplay;

// last poll status
typedef struct {
		int             status,
						crderror,					// card reader error status
						mdberror;					// interface error status
} CRDPollStatus;

/************************************************************** miscellaneous */

typedef unsigned char	InOutBuff[BUFFSIZE];  		// in-/ouput buffer

/******************************************************* common MDB functions */

#ifdef __cplusplus
extern "C" {
#endif

	// open and close MDB
 	int _stdcall OpenMDB ( int comnr );
	int _stdcall CloseMDB ( void );

	// reset all MDB devices
 	int _stdcall ResetMDBDevices ( void );

	// get/clear bill statistics
 	int _stdcall GetMDBBillStat ( MDBBillStat* pstat, int clear );

	// set bits of the auxiliary port
 	int _stdcall SetAuxPort ( char stat, char msk );

/*************************************************** coin validator fucntions */

	// open/close coin validator
 	int _stdcall OpenEMP ( void );
 	int _stdcall CloseEMP ( void );

	// reset coin validator
 	int _stdcall ResetEMP ( void );

	// get information about coin validator
 	int _stdcall GetEMPInfo ( EMPInfo* eip );

	// get more information about coin validator
 	int _stdcall GetEMPInfoEx ( EMPInfoEx* exp );

	// get current coin routing
 	int _stdcall GetEMPCurrent ( EMPCurrent* ecp );

	// set current coin routing
 	int _stdcall SetEMPCurrent ( EMPCurrent* ecp );

	// poll coin validator
 	int _stdcall PollEMP ( void );

	// get last poll status
 	int _stdcall GetLastEMPPoll ( EMPPollStatus* epp  );

    // get customer code
	int _stdcall GetEMPCustomer ( int* cdp );

/***************************************************** change giver functions */

	// open/close change giver
 	int _stdcall OpenCHG ( void );
 	int _stdcall CloseCHG ( void );

	// reset change giver
 	int _stdcall ResetCHG ( void );

	// get information about change giver
 	int _stdcall GetCHGInfo ( CHGInfo* cip );

	// get tube status
 	int _stdcall GetCHGTubeStat ( CHGTubeStat* ctp );

	// dispense a number of coins
 	int _stdcall DispenseCHGCoins ( CHGDispense* dcp );

	// dispense amount
 	int _stdcall DispenseCHGValue ( double* dvp, int force );

	// unlock coins
 	int _stdcall SetCHGCurrent ( CHGCurrent* ccp );

	// poll change giver
 	int _stdcall PollCHG ( void );

	// get last poll status
 	int _stdcall GetLastCHGPoll ( CHGPollStatus* cpp );

/*************************************************** bill validator functions */

	// open/close bill validator
 	int _stdcall OpenBLV ( void );
 	int _stdcall CloseBLV ( void );

	// reset
 	int _stdcall ResetBLV ( void );

	// get information about bill validator
 	int _stdcall GetBLVInfo ( BLVInfo* bip );

	// get stacker status
 	int _stdcall GetBLVStacker ( BLVStacker* bsp );

	// unlock bills
 	int _stdcall SetBLVCurrent ( BLVCurrent* bcp );

	// escrow or stack bill
 	int _stdcall ReturnBLVBill ( void );
 	int _stdcall StackBLVBill ( void );

	// poll bill validator
 	int _stdcall PollBLV ( void );

	// get last poll status
	int _stdcall GetLastBLVPoll ( BLVPollStatus* cpp );

/****************************************************** card reader functions */

	// open/close card reader
	int _stdcall OpenCRD ( void );
	int _stdcall OpenCRDEx ( int batt, res0, res1 );
	int _stdcall CloseCRD ( void );

	// reset
	int _stdcall ResetCRD ( void );

	// get information about card reader
	int _stdcall GetCRDInfo ( CRDInfo* cip );

	// initialize the card reader
	int _stdcall InitCRD ( void );

	// setup the card reader
	int _stdcall SetupCRD ( CRDSetup* csp );

	// get reader state
	int _stdcall GetCRDState ( void );

	// get display data
	int _stdcall GetCRDDisplay ( CRDDisplay* cdp );

	// get available credit
	double _stdcall GetCRDCredit ( void );

	// debit an amount
	int _stdcall DebitCRD ( double prc, double* dbtp );

	// cancel activities
	int _stdcall CancelCRD ( void );

	// poll the cardreader
	int _stdcall PollCRD ( void );

	// inquire last poll status
	int _stdcall GetLastCRDPoll ( CRDPollStatus* cpp );

    // Request vend
    int _stdcall VendRequestCRD ( float prc, *dbtp );

    // Vend success
    int _stdcall VendSuccessCRD ( void );

    // vend failure
    int _stdcall VendFailureCRD ( void );

#ifdef __cplusplus
}
#endif

#endif _WINMDB32_H

