/******************************************************************************/
/*                                                                            */
/*   Infos about currencies provided by the ECB                               */
/*                                                                            */
/*   wh Mnzprfer Berlin GmbH                                                */
/*   Teltower Damm 276                                                        */
/*   14167 Berlin                                                             */
/*                                                                            */
/*   Germany                                                                  */
/*                                                                            */
/*   Ausgust 2005                                                             */
/*                                                                            */
/******************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.Win32;

using System.Xml;
using System.Net;

namespace ECBCurrencyInfos
{
    public struct ECBCurrency
    {
        public ECBCurrency(string cd, int ncd, int dec, string desc, string accid, double rt, DateTime dt)
        {
            Code = cd;
            NumericCode = ncd;
            Decimals = dec;
            Description = desc;
            Decimals = dec;
            SelectorID = accid;
            Rate = rt;
            Date = dt;
        }
        public string Code;
        public int NumericCode;
        public int Decimals;
        public string Description;
        public string SelectorID;
        public double Rate;
        public DateTime Date;
    }

    public enum ECBCurrencyErrors
    {
        Ok,
        XMLNotFound,
        XMLFormat,
        SaveError,
        LoadError,
        OldFile,
    }

    class ECBCurrencyList
    {
        // Constructor, Destructor
        public ECBCurrencyList()
        {
            if (LoadCurrencyList() != ECBCurrencyErrors.Ok)
                InitCurrencies();
        }
        ~ECBCurrencyList()
        {
            SaveCurrencyList();
        }

        public bool CheckConnection()
        {
            XmlTextReader xmlReader;

            try
            {
                xmlReader = new XmlTextReader(WebAddress);
            }
            catch (WebException)
            {
                return false;
            }
            try
            {
                xmlReader.Read();
            }
            catch (WebException)
            {
                xmlReader.Close();
                return false;
            }
            xmlReader.Close();
            return true;
        }

        public ECBCurrencyErrors UpdateRates()
        {
            int i, j;
            string tcurr;
            DateTime tdate = new DateTime(2005, 08, 26);
            decimal trate;

            XmlTextReader xmlReader;

            try
            {
                xmlReader = new XmlTextReader(WebAddress);
            }
            catch (WebException) 
            {
                InitCurrencies();
                return ECBCurrencyErrors.XMLNotFound;
            }
            try
            {
                while (xmlReader.Read())
                {
                    if (xmlReader.Name != "")
                    {
                        if (xmlReader.Name == "gesmes:name")
                        {
                            source = xmlReader.ReadString();
                        }
                        for (i = 0; i < xmlReader.AttributeCount; i++)
                        {
                            if (xmlReader.Name == "Cube")
                            {
                                switch (xmlReader.AttributeCount)
                                {
                                    case 1:
                                        {
                                            xmlReader.MoveToAttribute("time");
                                            tdate = DateTime.Parse(xmlReader.Value);
                                            break;
                                        }
                                    case 2:
                                        {
                                            xmlReader.MoveToAttribute("currency");
                                            tcurr = xmlReader.Value;
                                            xmlReader.MoveToAttribute("rate");
                                            trate = decimal.Parse(xmlReader.Value.Replace(".", ","));

                                            for (j = 0; j < currlist.Length; j++)
                                            {
                                                if (currlist[j].Code == tcurr)
                                                {
                                                    if (tdate > currlist[j].Date)
                                                    {
                                                        currlist[j].Date = tdate;
                                                        currlist[j].Rate = (double)trate;
                                                    }
                                                    break;
                                                }
                                            }
                                            break;
                                        }
                                }
                            }
                        }
                    }
                }
            }
            catch (WebException)
            {
                xmlReader.Close();
                InitCurrencies();
                return ECBCurrencyErrors.XMLNotFound;
            }
            xmlReader.Close();
            SaveCurrencyList();
            return ECBCurrencyErrors.Ok;
        }

        public ECBCurrency RetrieveCurrency(string accid)
        {
            int i;
            for (i = 0; i < currlist.Length; i++)
                if (currlist[i].SelectorID == accid) return currlist[i];
            return nocurr;
        }
        public ECBCurrency GetCurrency(int idx)
        {
            if ((idx >= 0) && (idx < currlist.Length))
            {
                return currlist[idx];
            }
            else
            {
                return nocurr;
            }
        }
        public int Count
        {
            get { return currlist.Length; }
        }
        public string Source
        {
            get { return source; }
        }

        #region Internal variables, methods etc.
        internal string WebAddress = "http://www.ecb.int/stats/eurofxref/eurofxref-hist.xml";
        internal const string REGISTRY_PATH = "Software\\whberlin\\ECBCurrencyList\\";
        //string WebAddress = "E|/Temp/eurofxref-hist.xml";
        internal string currflnm = "ECBExchangeRates.dat";
        internal ECBCurrency[] currlist = new ECBCurrency[0];
        internal string source = "";
        internal RegistryKey creg;

        // Return value for unknown currency
        internal ECBCurrency nocurr = new ECBCurrency("***", 0, 2, "Unknown currency", "**", 1, DateTime.Now);
 
        // Initialisation of currencies
        // Exchange rates as of 26.08.2005
        internal void InitCurrencies()
        {
            DateTime defdate = new DateTime(2005, 08, 26);
            currlist = new ECBCurrency[]
            {
                new ECBCurrency("EUR", 978, 2, "European Union", "EU", 1, new DateTime(2002, 01, 01)),
                new ECBCurrency("AUD", 036, 2, "Australian Dollar", "AU", 1.6209, defdate),
                new ECBCurrency("BGN", 975, 2, "Bulgarian Lev (new)", "BG", 1.9557, defdate),
                new ECBCurrency("CAD", 124, 2, "Canadian Dollar", "CA", 1.4709, defdate),
                new ECBCurrency("CHF", 756, 2, "Swiss Franc", "CH", 1.5467, defdate),
                new ECBCurrency("CNY", 156, 2, "Chinese Yuan Renminbi", "CN", 9.9644, defdate),
                new ECBCurrency("CYP", 196, 2, "Cyprus Pound", "CY", 0.5729, defdate),
                new ECBCurrency("CZK", 203, 2, "Czech Koruna", "CZ", 29.634, defdate),
                new ECBCurrency("DKK", 208, 2, "Danish Krone", "DK", 7.4582, defdate),
                new ECBCurrency("EEK", 233, 2, "Estonian Kroon", "EE", 15.6466, defdate),
                new ECBCurrency("GBP", 826, 2, "Pound Sterling", "GB", 0.68185, defdate),
                new ECBCurrency("HKD", 344, 2, "Hong Kong Dollar", "HK", 9.5617, defdate),
                new ECBCurrency("HRK", 191, 2, "Croatian Kuna", "HR", 7.3845, defdate),
                new ECBCurrency("HUF", 348, 0, "Hungarian Forint ", "HU", 245.59, defdate),
                new ECBCurrency("IDR", 360, 0, "Indonesian Rupiah", "ID", 12780.82, defdate),
                new ECBCurrency("ISK", 352, 2, "Iceland Krona", "IS", 78.06, defdate),
                new ECBCurrency("JPY", 392, 0, "Japanese Yen", "JP", 134.93, defdate),
                new ECBCurrency("KRW", 410, 0, "South Korean Won", "KR", 1264.11, defdate),
                new ECBCurrency("LTL", 440, 2, "Lithuanian Litas", "LT", 3.4528, defdate),
                new ECBCurrency("LVL", 428, 2, "Latvian Lats", "LV", 0.696, defdate),
                new ECBCurrency("MTL", 470, 2, "Maltese Lira", "MT", 0.4293, defdate),
                new ECBCurrency("MYR", 458, 2, "Malaysia Ringgit", "MY", 4.6341, defdate),
                new ECBCurrency("NOK", 578, 2, "Norvegian Krone", "NO", 7.926, defdate),
                new ECBCurrency("NZD", 554, 2, "New Zealand Dollar", "NZ", 1.7622, defdate),
                new ECBCurrency("PHP", 608, 2, "Philipines Peso", "PH", 68.833, defdate),
                new ECBCurrency("PLN", 985, 2, "Polish Zloty", "PL", 4.0583, defdate),
                new ECBCurrency("RON", 946, 2, "Romanian Leu (new)", "RO", 3.5359, defdate),
                new ECBCurrency("RUB", 810, 2, "Russian Ruble", "RU", 35.028, defdate),
                new ECBCurrency("SEK", 752, 2, "Swedish Krona", "SE", 9.3291, defdate),
                new ECBCurrency("SGD", 702, 2, "Singapore Dollar", "SG", 2.0576, defdate),
                new ECBCurrency("SIT", 705, 2, "Slovenian Tolar", "SI", 239.52, defdate),
                new ECBCurrency("SKK", 703, 2, "Slovak Koruna", "SK", 38.823, defdate),
                new ECBCurrency("THB", 764, 2, "Thailand Baht", "TH", 50.481, defdate),
                new ECBCurrency("TRY", 949, 2, "Turkish Lira (new)", "TR", 1.6742, defdate),
                new ECBCurrency("USD", 840, 2, "US Dollar", "US", 1.2307, defdate),
                new ECBCurrency("ZAR", 710, 2, "South African Rand", "ZA", 7.9522, defdate),
                new ECBCurrency("---", 000, 2, "Token", "TK", 0.0000, new DateTime(2000, 01, 01)),
            };
            LoadCurrencyList();
        }

        // Save current currency list
        internal ECBCurrencyErrors SaveCurrencyList()
        {
            int i;

            try
            {
                for (i = 0; i < currlist.Length; i++) if (currlist[i].Rate > 0.00001)
                {
                    creg = Registry.CurrentUser.CreateSubKey(REGISTRY_PATH + currlist[i].Code);
                    creg.SetValue("NumericCode", string.Format("{0:d3}", currlist[i].NumericCode), RegistryValueKind.String);
                    creg.SetValue("Decimals", string.Format("{0:d1}", currlist[i].Decimals), RegistryValueKind.String);
                    creg.SetValue("Description", currlist[i].Description, RegistryValueKind.String);
                    creg.SetValue("Rate", string.Format("{0:f4}", currlist[i].Rate), RegistryValueKind.String);
                    creg.SetValue("Date", string.Format("{0:d}", currlist[i].Date), RegistryValueKind.String);
                    creg.SetValue("SelectorID", currlist[i].SelectorID, RegistryValueKind.String);
                    creg.Close();
                }

            }
            catch (System.Exception) { return ECBCurrencyErrors.SaveError; };

            return ECBCurrencyErrors.Ok;
        }
        // Try to load currency list
        internal ECBCurrencyErrors LoadCurrencyList()
        {
            int i, j;
            string[] scodes;

            try
            {
                creg = Registry.CurrentUser.OpenSubKey(REGISTRY_PATH);
                scodes = creg.GetSubKeyNames();
                creg.Close();
                for (i = 0; i < scodes.Length; i++)
                {
                    for (j = 0; j < currlist.Length; j++)
                    {
                        if (currlist[j].Code == scodes[i])
                        {
                            creg = Registry.CurrentUser.OpenSubKey(REGISTRY_PATH + "\\" + scodes[i]);
                            currlist[j].Rate = Convert.ToDouble((string)creg.GetValue("Rate", 1));
                            currlist[j].Date = Convert.ToDateTime((string)creg.GetValue("Date", 1));
                            creg.Close();
                        }
                    }
                }
            }
            catch (System.Exception) { return ECBCurrencyErrors.LoadError; };
            return ECBCurrencyErrors.LoadError;
        }
        #endregion
    }
}
