using System; 
using System.Runtime.InteropServices; 
using System.Windows.Forms; 
using System.Windows; 
using System.Collections; 

namespace SystemMenus
{
    public class SystemMenu : ArrayList
    {
        #region MenuItem flags
        public const int BYPOSITION = 0x0400; 
        public const int REMOVE     = 0x1000; 
        public const int CHECKED    = 0x0008; 
        public const int APPEND     = 0x0100; 
        public const int SEPERATOR  = 0x0800; 
        public const int GRAYED     = 0x0001; 
        public const int DISABLED   = 0x0002; 
        public const int BITMAP     = 0x0004; 
        public const int RADIOCHECK = 0x0200; 
        public const int BREAK      = 0x0040; 
        public const int BARBREAK   = 0x0020;
        #endregion

        #region DLL Imports
        [DllImport("user32.dll")] 
        private static extern int GetSystemMenu(int HWND, int reset); 
        [DllImport("user32.dll")] 
        private static extern int AppendMenu(int MenuHandle, int Props, int FlagsW, string text); 
        [DllImport("user32.dll")] 
        private static extern int RemoveMenu(int MenuHandle, int pos, int Flags); 
        [DllImport("user32.dll")] 
        private static extern int GetMenuItemID(int Menuhandle, int pos); 
        [DllImport("user32.dll")] 
        private static extern int ModifyMenu(int MHandle, int pos,int flags,int newPos,string text);
        #endregion

        private Form form; 
        private int SystemMenuHandle; 
            
        public int Handle { 
            get { return SystemMenuHandle; } 
        } 

        public SystemMenu(Form f) : base(0) { 
            form = f; 
            SystemMenuHandle = GetSystemMenu(form.Handle.ToInt32(), 0); 
        } 
            
        public void Add(SystemMenuItem MI) { 
            base.Add(MI); 
            if(MI.Text == "-") { 
                AppendMenu(SystemMenuHandle,SystemMenu.SEPERATOR,MI.MenuID,null); 
            } else { 
                AppendMenu(SystemMenuHandle,MI.Flags,MI.MenuID,MI.Text); 
            } 
        } 

        public void Remove(SystemMenuItem MI) { 
            base.Remove(MI); 
            RemoveMenu(SystemMenuHandle, MI.MenuID, 0); 
        } 

        public void ModifyMenuPosition(int pos, int flags, string text) { 
            ModifyMenu(this.Handle, pos, flags|SystemMenu.BYPOSITION, pos, text); 
        } 

        public new SystemMenuItem this[int index] { 
            get { return (SystemMenuItem)base[index]; } 
            set { 
                if(value!=null) { 
                SystemMenuItem MI = (SystemMenuItem)value; 
                ModifyMenu(this.Handle, this[index].MenuID, MI.Flags, MI.MenuID, MI.Text); } 
                base[index] = (object)value; 
            } 
        } 

    }// end of class SystemMenu 

    public class SystemMenuItem : MenuItem
    {
        #region DLL Imports
        [DllImport("user32.dll")] 
        private static extern int GetSystemMenu(int HWND); 
        [DllImport("user32.dll")] 
        private static extern int AppendMenu(int MenuHandle, int Props, int FlagsW, string text); 
        [DllImport("user32.dll")] 
        private static extern int RemoveMenu(int MenuHandle, int pos, int Flags); 
        [DllImport("user32.dll")] 
        private static extern int GetMenuItemID(int Menuhandle, int pos); 
        [DllImport("user32.dll")] 
        private static extern int ModifyMenu(int MHandle, int pos,int flags,int newPos,string text); 
        [DllImport("user32.dll")] 
        private static extern int CheckMenuItem(int HMenu, int pos, int flags);
        #endregion

        private int flags = 0; 
        public int Flags { 
            get { return flags; } 
        } 

        private SystemMenu menu;     
        public SystemMenuItem(string text, SystemMenu SM) { 
            base.Text = text; 
            menu = SM; 
            if(text == "-") 
                this.flags = SystemMenu.SEPERATOR; 
        } 

        public new int MenuID { 
            get { return base.MenuID; } 
        } 

        public SystemMenuItem CloneMenu(int should_be_null) { 
            should_be_null = 0; 
            return new SystemMenuItem(this.Text, menu); 
        } 

        public new bool Checked { 
            get { return base.Checked; } 
            set { 
                base.Checked = value; 
                if(base.Checked) { 
                    flags = (flags|SystemMenu.CHECKED); 
                    CheckMenuItem(menu.Handle, this.MenuID, flags); 
                } else { 
                    flags = (flags&(~SystemMenu.CHECKED)); 
                    CheckMenuItem(menu.Handle, this.MenuID, flags); 
                }     
            } 
        } 

        public new string Text { 
            get { return base.Text; } 
            set { 
                base.Text = value; 
                ModifyMenu(menu.Handle, this.MenuID, this.flags, this.MenuID, base.Text); 
            } 
        } 
            
        public new bool Break { 
            get { return base.Break; } 
            set { 
                base.Break = value; 
                if(base.Break) { 
                    flags = flags|SystemMenu.BREAK; 
                    ModifyMenu(menu.Handle, this.MenuID, this.flags, this.MenuID, base.Text); 
                } else { 
                    flags = flags&(~SystemMenu.BREAK); 
                    ModifyMenu(menu.Handle, this.MenuID, this.flags, this.MenuID, base.Text); 
                } 
            } 
        } 

        public new bool BarBreak { 
            get { return base.BarBreak; } 
            set { 
                base.BarBreak = value; 
                if(base.BarBreak) { 
                    flags = flags|SystemMenu.BARBREAK; 
                    ModifyMenu(menu.Handle, this.MenuID, this.flags, this.MenuID, base.Text); 
                } else { 
                    flags = flags&(~SystemMenu.BARBREAK); 
                    ModifyMenu(menu.Handle, this.MenuID, this.flags, this.MenuID, base.Text); 
                } 
            } 
        } 

            
    }//end of class SystemMenuItem 
}
