using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

using whSelectorCommunication;
using ECBCurrencyInfos;

using SystemMenus;

namespace USBCoinSelector
{
    // Credits per ID
    public struct SelCredit
    {
        public string ID;
        public double Total;
    }

    public partial class FrmMain : Form
    {

        public FrmMain()
        {
            int i;

            InitializeComponent();
            CurrentDevs = new whSelectorList();
            SelWindows = new SelectorForm[0];
            CurrConfig = new USBSelConfig();
            Currencies = new ECBCurrencyList();

            GlobalCredits = new SelCredit[Currencies.Count];
            SelCreditCount = 0;

            cbBaseCurrency.Items.Clear();
            for (i = 0; i < Currencies.Count; i++)
            {
                cbBaseCurrency.Items.Add(Currencies.GetCurrency(i).Code);
            }
            cbBaseCurrency.SelectedIndex = 0;
            tbTotal.Text = "";

            sysmen = new SystemMenu(this);
            miSep00 = new SystemMenuItem("-", sysmen);
            sysmen.Add(miSep00);
            miToTray = new SystemMenuItem("Minimize To Tray", sysmen);
            sysmen.Add(miToTray);
            miToTray.Click += new EventHandler(miToTray_Click);
            INotify.Visible = false;
        }

        // Window
        private SystemMenu sysmen;
        private SystemMenuItem miToTray, miSep00;

        // Credit and currencies
        public SelCredit[] GlobalCredits;
        public int SelCreditCount;
        public void ShowDevicesStates()
        {
            int i;
            for (i = 0; i < SelWindows.Length; i++)
            {
                lvDevices.Items[i].SubItems[4].Text = SelWindows[i].Status.ToString();
            }
            Update();
        }
        public void ShowCurrencies()
        {
            int i;

            ECBCurrency ccurr;
            lvCurrencies.Items.Clear();
            for (i = 0; i < Currencies.Count; i++)
            {
                ccurr = Currencies.GetCurrency(i);
                string[] newItems = new string[] 
                {
                    "",
                    ccurr.Code,
                    string.Format("{0:d3}", ccurr.NumericCode),
                    ccurr.Description,
                    ccurr.SelectorID,
                    string.Format("{0:f4}", ccurr.Rate),
                    string.Format("{0:d}", ccurr.Date),
                };
                ListViewItem lvi = new ListViewItem(newItems);
                lvCurrencies.Items.Add(lvi);
            }
            cbBaseCurrency.SelectedIndex = (cbBaseCurrency.Items.IndexOf(CurrConfig.BaseCurrency));
        }
        public void ClearCredits()
        {
            int i;

            for (i = 0; i < SelCreditCount; i++) GlobalCredits[i].Total = 0;
        }
        public void ShowCredits()
        {
            int i;
            ECBCurrency ccurr, bcurr;

            lvCredits.Items.Clear();
            TotalCredit = 0;
            bcurr = Currencies.GetCurrency(cbBaseCurrency.SelectedIndex);
            for (i = 0; i < SelCreditCount; i++)
            {
                ccurr = Currencies.RetrieveCurrency(GlobalCredits[i].ID);
                if (ccurr.Rate > 0.00001)
                {
                    string[] newItems = new string[] 
                    {
                        "",
                        string.Format("{0:f" + ccurr.Decimals.ToString() + "}", GlobalCredits[i].Total),
                        ccurr.Code,
                    };
                    ListViewItem lvi = new ListViewItem(newItems);
                    lvCredits.Items.Add(lvi);
                    TotalCredit += GlobalCredits[i].Total * bcurr.Rate / ccurr.Rate;
                }
            }
            if (SelCreditCount > 0)
                tbTotal.Text = string.Format("{0:f" + bcurr.Decimals.ToString() + "}", TotalCredit);
            else tbTotal.Text = "";
            if (TotalCredit > 0.0001)
            {
                INotify.Text = "Credit: " + tbTotal.Text + " " + cbBaseCurrency.Text;
            }
            else
            {
                INotify.Text = Text;
            }
        }

        private void EnableButtons(bool status)
        {
            btnOpen.Enabled = false;
            btnClearCredits.Enabled = status;
            btnQuit.Enabled = status;
            btnSearch.Enabled = status;
            btnUpdateRates.Enabled = status && Currencies.CheckConnection();
            if (status) Cursor.Current = Cursors.Arrow;
            else Cursor.Current = Cursors.WaitCursor;
        }

        // Handling the tray icon
        private void RestoreWindow()
        {
            int i;
            Show();
            for (i = 0; i < SelWindows.Length; i++)
            {
                if (SelWindows[i].WasVisible) SelWindows[i].Show();
            }
            INotify.Visible = false;
        }
        private void miToTray_Click(object sender, EventArgs e)
        {
            int i;
            Hide();
            for (i = 0; i < SelWindows.Length; i++)
            {
                SelWindows[i].WasVisible = SelWindows[i].Visible;
                SelWindows[i].Hide();
            }
            INotify.Visible = true;
        }
        private void tsmiRestore_Click(object sender, EventArgs e)
        {
            RestoreWindow();
        }
        private void tsmiClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private bool DevicesActive()
        {
            bool DevsActive;
            int i;
            DevsActive = false;
            for (i = 0; i < SelWindows.Length; i++)
            {
                if (SelWindows[i].Status != SelectorStates.Closed)
                {
                    DevsActive = true;
                    break;
                }
            }
            return DevsActive;
        }
        private void CloseDevices()
        {
            int i;
            for (i = 0; i < SelWindows.Length; i++)
            {
                if (CurrentDevs.Selectors[i].IsOpen) CurrentDevs.Selectors[i].CloseComm();
                SelWindows[i].Close();
            }
        }

        private void FrmMain_Load(object sender, EventArgs e)
        {
            CurrConfig.Load();
            CurrConfig.LoadWinPos(this, "Main", false);
            ShowCurrencies();
            label03.Text = "Version " + Application.ProductVersion;
        }
        private void FrmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = !btnQuit.Enabled;
            if (!e.Cancel)
            {
                if (DevicesActive())
                {
                    e.Cancel = MessageBox.Show(
                                this,
                                "There are open devices. Close application anyway?\t",
                                Text,
                                MessageBoxButtons.OKCancel,
                                MessageBoxIcon.Exclamation,
                                MessageBoxDefaultButton.Button2) == DialogResult.Cancel;
                }
                if (!e.Cancel)
                {
                    CurrConfig.SaveWinPos(this, "Main");
                    CurrConfig.Save();
                }
            }
        }
        private void BtnClose_Click(object sender, EventArgs e)
        {
            Close();
        }
        private void FrmMain_Resize(object sender, EventArgs e)
        {
            int i;

            for (i = 0; i < SelWindows.Length; i++)
                SelWindows[i].WindowState = this.WindowState;
        }

        // List of active USB coin selectors and related windows
        private whSelectorList CurrentDevs;
        private SelectorForm[] SelWindows;
        // Currencies
        private ECBCurrencyList Currencies;
        private double TotalCredit;
        // Configuration
        private USBSelConfig CurrConfig;

        private void BtnSearch_Click(object sender, EventArgs e)
        {
            int i;

            EnableButtons(false);
            if (DevicesActive())
            {
                if (MessageBox.Show(
                            this,
                            "There are open devices. Search anyway?\t",
                            Text,
                            MessageBoxButtons.OKCancel,
                            MessageBoxIcon.Exclamation,
                            MessageBoxDefaultButton.Button2) == DialogResult.Cancel)
                {
                    EnableButtons(true);
                    return;
                }
            }
            CloseDevices();
            label02.Text = "";
            lvDevices.Items.Clear();
            for (i = 0; i < SelWindows.Length; i++) SelWindows[i].Close();
            ShowCredits();
            Update();

            SelWindows = new SelectorForm[0];
            CurrentDevs.SearchSelectors();

            switch (CurrentDevs.Count)
            {
                case 0:
                    {
                        label02.Text = "No devices found.";
                        break;
                    }
                case 1:
                    {
                        label02.Text = "One device found.";
                        break;
                    }
                default:
                    {
                        label02.Text = CurrentDevs.Count.ToString() + " devices found.";
                        break;
                    }
            }

            if (CurrentDevs.Count > 0)
            {
                SelWindows = new SelectorForm[CurrentDevs.Count];
                for (i = 0; i < CurrentDevs.Count; i++)
                {
                    SelWindows[i] = new SelectorForm(CurrentDevs.Selectors[i], CurrConfig, this, i + 1);
                    if (CurrentDevs.Selectors[i].OpenComm() == whSelErrors.Ok)
                    {
                        string[] newItems = new string[] 
                        {
                            "",
                            string.Format("{0:d}", i + 1),
                            string.Format("{0:d8}", CurrentDevs.Selectors[i].SerialNumber),
                            CurrentDevs.Selectors[i].SoftwareRevision,
                            SelWindows[i].Status.ToString(),
                        };
                        ListViewItem lvi = new ListViewItem(newItems);
                        lvDevices.Items.Add(lvi);
                        CurrentDevs.Selectors[i].CloseComm();
                    }
                    else
                    {
                        string[] newItems = new string[] 
                        {
                            "",
                            string.Format("{0:d}", i + 1),
                            "Error",
                            "",
                            "",
                        };
                        ListViewItem lvi = new ListViewItem(newItems);
                        lvDevices.Items.Add(lvi);
                    }
                }
            }
            EnableButtons(true);        
        }
        private void lvDevices_DoubleClick(object sender, EventArgs e)
        {
            int devidx;

            devidx = lvDevices.FocusedItem.Index;
            if ((CurrentDevs.Count > 0) && (devidx < CurrentDevs.Count))
            {
                if (SelWindows[devidx].Visible)
                {
                    SelWindows[devidx].BringToFront();
                }
                else
                {
                    SelWindows[devidx].Show();
                }
            }
        }
        private void lvDevices_Click(object sender, EventArgs e)
        {
            int devidx;

            devidx = lvDevices.FocusedItem.Index;
            btnOpen.Enabled = ((CurrentDevs.Count > 0) && (devidx < CurrentDevs.Count));
        }

        private void btnClearCredits_Click(object sender, EventArgs e)
        {
            EnableButtons(false);
            ClearCredits();
            ShowCredits();
            EnableButtons(true);
        }

        private void btnUpdateRates_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this,
                                "Updating exchange rates may take a couple of minutes.\n\nDownload rates from the ECB website now?",
                                Text,
                                MessageBoxButtons.OKCancel,
                                MessageBoxIcon.Question) == DialogResult.OK)
            {
                EnableButtons(false);
                Update();
                switch (Currencies.UpdateRates())
                {
                    case ECBCurrencyErrors.Ok:
                    {
                        label01.Text = "Source: " + Currencies.Source;
                        break;
                    }
                    case ECBCurrencyErrors.XMLFormat:
                    {
                        label01.Text = "Update failed: XML source corrupted.";
                        break;
                    }
                    case ECBCurrencyErrors.XMLNotFound:
                    {
                        label01.Text = "Update failed: XML source not found.";
                        break;
                    }
                }
                ShowCurrencies();
                ShowCredits();
                EnableButtons(true);
            }
        }

        private void cbBaseCurrency_SelectedIndexChanged(object sender, EventArgs e)
        {
            CurrConfig.BaseCurrency = cbBaseCurrency.Items[cbBaseCurrency.SelectedIndex].ToString();
            ShowCredits();
        }

     }
}